#ifndef COMMON_H
#define COMMON_H

#define MAX_HOSTNAME_LENGTH 256
#define MAC_ADDR_LENGTH 6

#define DEFAULT_SEND_PORT 3000
#define DEFAULT_RECV_PORT 4000
#define MAXBUFLENGTH 10000

#include <string.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <unistd.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>

#ifndef INADDR_NONE
#  define INADDR_NONE (-1)
#endif

///A Packet class
/**
 * Packet is an entity which contains a set of ordered information bits. 
 * This packet object only carries a pointer to
 * payload information, not any socket address information. 
 */
class Packet {
 
public:
  /**
   *  Default payload size is 512 Bytes
   */
  static const int DEFAULT_PAYLOAD_SIZE = 512;
  Packet();
  Packet(int buffer_length);
  int fillPayload(int size, char *inputstream);  
   /** get a pointer to the payload
    * @return  a char pointer
    */
  inline char* getPayload(){ return payload_;} 
  void setPayloadSize(int size);
  /**get the size of packet buffer where payload is stored.
   *
   */
  inline int getBufferSize(){return length_;}
 /** get the size of the packet 
  *  
  */
  inline int getPayloadSize(){ return size_;}
 
 
private:               
      int size_;  ///< Packet length in Bytes
      int length_; ///< Maximum allocated Size of Payload buffer. it is no less than the size_
      char* payload_; ///< Payload Pointer.

};

/// An address class
/**
 *  Address Class is to handle addresses.
 *  For normal sockets, the address used will be a combination of IP address and port.
 *  In Socket Programming, IP address itself is usually not enough to distinguish an connection, port # is also needed.
 *  For PF_PACKET sockets, the address used is MAC address (HW address). So, we also put macaddr_ as a member variable.
 */

class Address
{
  
 public:
  /// Constructor    
  Address();
  /// Alternative construcor with parameters
  Address(const char* hostname, short port);
   
  /**
   * Check if an address has already been set or remain uninitialized
   *
   */ 
  inline bool isSet() { return (hostname_[0] != '\0' && port_ >= 0); }
  /**
   * set the port # of the Address
   */
  inline void setPort(const short port){ port_ = port; }
  /// get the port #
  inline short getPort(){ return port_; }
  /// set the hostname
  /** use strcpy function to duplicate a string
   *
   */
  inline void setHostname(const char* hostname) { 
    if (hostname == NULL) hostname_[0] = '\0'; else strcpy(hostname_, hostname); }
  /// get the hostname string pointer
  inline char* getHostname() {return hostname_;} 
  /// get the MAC address
  inline unsigned char* getHWAddr() { return macaddr_;} 
  void setHWAddr(unsigned char* hwaddr); 
  void setHWAddrFromColonFormat(const char* colon_seperated_macaddr);
  char * convertHWAddrToColonFormat();
  /// function to clone this address
  Address *clone();
  /**
   *Compare whether the two normal "name+port" address is same or not
   */
  inline bool isSame(Address* addr)
  {
    if ( port_==  addr->getPort()) return false;
    if ( strcmp( hostname_, addr->getHostname() ) == 0 ) return true;
    return false;
  } 
  bool isSameMACAddr(Address* addr);

private:
  char   hostname_[MAX_HOSTNAME_LENGTH]; ///< both hostname and ipaddress format (10.0.0.1) could be given as a string
  short port_;  ///< port number for UDP or TCP (Transport layer)
  char * ipaddr_;  ///<optional use... ignore.... 
  unsigned char   macaddr_[MAC_ADDR_LENGTH]; ///< optional use for Ethernet Socket
  
};

/// Port class abstacts functions of communication interfaces
/** 
 * Port is an abstract class for the interface to send/receive a packet, whether UDP, TCP Socket, or IP raw socket.
 * The common funcitons for a port are defiend here:
 * - init
 * - setSocketAddress
 *
    Port is also associated with a pair of address. One Port send to one Address only,
    no matter the address is unicast or broadcast.
*/


class Port
{
 public:
  /// Constructor
  Port();
  /// Deconstructor
  virtual ~Port(){}
   /**
    * Function to initialize the port
    */
  virtual void init()=0;
  /// set the port's own address
  void setAddress(Address *addr);
  /// set the address of the port at the other end of communication link
  void setRemoteAddress( Address *daddr);
  /// get the address of the port at the other end of communication link
  inline Address *getRemoteAddr(){return &itsaddr_;}
  /// close the port
  inline void closePort(){close(sockfd_);}
 protected:
  /// cast an Address to socket address format
  struct sockaddr* setSockAddress(Address *addr, struct sockaddr_in *address);
  /// cast a socket address to normal address format
  void decodeSockAddress ( Address *addr, struct sockaddr_in *address);
  /// set hostname of local address
  inline void setHostname(const char* hostname){ myaddr_.setHostname(hostname);}
  /// set port of local address
  inline void setPort(const short port){ myaddr_.setPort(port);}
  /// set hostname of remote address
  inline void setRemoteHostname(const char* hostname){itsaddr_.setHostname(hostname);}
  /// set port no of a remote address
  inline void setRemotePort(const short port){itsaddr_.setPort(port); }
  /// get the socket file descriptor
  inline int getSock(){ return sockfd_;}
 
 protected:
  Address myaddr_;  ///< The default address of mine 
  Address itsaddr_; ///< The default address of the other end of communication link.
  struct sockaddr_in mySockAddress_;  ///<IN UDP. this will be INADDR_ANY, not really my IP addr
  struct sockaddr_in dstSockAddress_;  ///< for every outgoing packet of a connection, the destination address.
  int sockfd_; ///<socket file descriptor
};

/// SendingPort is an subclass of Port for sending purpose
/** SendingPort is an subclass of Port for sending purpose
 *
 */


class SendingPort : public Port
{
 public:
  SendingPort();
  /// Another constructor with local address given
  SendingPort(char* hostname, short port);
  ///Deconstructor
  virtual ~SendingPort(){}
   /**
    * Function to initialize the port
    */
  void init();
   /**
    * Function to send a packet.
    * The default socket file descriptor will always be used for send()
    * only a sockfd used by the port.
    */
  void sendPacket(Packet *pkt);
  /**
   * toggle broadcast option on
   */
  inline void setBroadcast(){bcastflag_ = 1;}
  /**
   * toggle broadcast option off
   */
  inline void setBroadcastOff(){bcastflag_ = 0;}

 protected:
  /// this flag indicates the port ought to broadcast or unicast a packet. 
  /** 
   * If broadcast, an broadcast IP address (192.168.255.255, etc) need to be supplied
   */
  int bcastflag_; 
};

/// A Receiving Port Class
/** 
 * ReceivingPort is an abstract class for the interface to receive a packet.
 * The main function for the receiving port:
 * - initialize
 * - Receive Packet
 *
 */

class ReceivingPort : public Port
{
 public:
  ReceivingPort();
  virtual ~ReceivingPort(){}
  void init();
  ///The main receive function of receiving port to receive a single packet
  Packet* receivePacket(); 
  
 protected:
   /**  This pointer points to the packet.
    *   This packet is just received.
    */
   Packet *pkt_;   
   char *tmpBuffer_;  ///<temporary buffer for packets
};


#endif


